//=============================================================================
//                                                            FFCal_v01.mq4
//                                            Copyright  2006, Derk Wehler
// Written in cooperation with:                 http://www.forexfactory.com
//
//
// This "indicator" calls DLLs to fetch a special XML file from the 
// ForexFactory web site.  It then parses it and writes it out as a .CSV 
// file, which it places in the folder: experts/files so that IsNewsTime() 
// can use that file to tell if it is near announcement time.
//
// It does this once when it starts up and once per hour in case there have 
// been any updates to the annoucement calendar.
//
// In order for this to work correctly and synchronize with your broker's 
// server time, you must enter the proper value for "TimeZone".  This is 
// entered relative to GMT.  For example, my broker's server time is GMT+1, 
// so I have the default set to 1.  If your broker uses GMT+3, enter 3.
// If your broker uses GMT-2, then enter -2.
//
//
//
//=============================================================================
//                             OTHER CREDIT DUE
//      (For GrebWeb and LogUtils functionality (see end of this flie)
//
//  2/14/2007:  Robert Hill added code for using text objects instead
//              of Comment() for easier reading
//
//  2/25/2007:  Paul Hampton-Smith for his TimeZone DLL code
//=============================================================================
//+---------------------------------------------------------------------------+
//|                                                               WinInet.mqh |
//|                                                        Paul Hampton-Smith |
//|                                                        paul1000@pobox.com |
//|                                                                           |
//| This include has been adapted from the groundbreaking GrabWeb script      |
//| developed by Adbi that showed me how to import functions from wininet.dll |
//|---------------------------------------------------------------------------|
//|                                                               grabweb.mq4 |
//|                                                    Copyright  2006, Abhi |
//|                                         http://www.megadelfi.com/experts/ |
//|                                     E-mail: grabwebexpert{Q)megadelfi.com |
//|                                fix my e-mail address before mailing me ;) |
//+---------------------------------------------------------------------------+
/*
		IDEAS FOR ADDITIONS:
		
*   A text lable of what the news was and when it was, e.g
    "USD CPI m/m @ [broker time here]" at the top or bottom within the shaded area.
    
*   Label the number of pips it went in its initial direction and how much it 
	retraced...of course you'd have to come up with a convention for measuring 
	movement.  Or you could label what price was just prior to news then labels 
	the next two price exteremes.  I think it would be nice to have the numbers 
	right under/over the candle that produced the extremes for whatever timeframe 
	you are in.

*   Label the volume count for the candle that included the news in it...whatever 
	timeframe your in.  So I could switch from 1 min. to 5 min. to 15 min. 
	timeframes and get different vol. counts for the bar that included the news.
	
*/

#property copyright "Copyright  2006, Derk Wehler"
#property link      "http://www.forexfactory.com"

#property indicator_chart_window
#property indicator_buffers 8

#define TITLE		0
#define COUNTRY	1
#define DATE		2
#define TIME		3
#define IMPACT		4
#define FORECAST	5
#define PREVIOUS	6

//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================
//====================================================================================

extern bool    Text              = true;
extern bool    TextOutput        = true;
extern bool 	IncludeHigh 		= true;
extern bool 	IncludeMedium 		= true;
extern bool 	IncludeLow 			= true;
extern bool 	IncludeSpeaks 		= true; 	// news items with "Speaks" in them have different characteristics
extern int		Alert1MinsBefore	= 400;//-1;		// Set to -1 for no Alert

extern bool 	EnableLogging 		= false; 	// Perhaps remove this from externs once its working well
extern bool		ShowNextTwoEvents	= true;
extern int 		TxtSize 			= 10;
extern color 	TxtColorTitle 		= LightGray;
extern color 	TxtColorNews 		= DeepSkyBlue;
extern color 	TxtColorImpact 		= Red;
extern color 	TxtColorPrevious 	= Orange;
extern color 	TxtColorForecast 	= Lime;

string         comment_str1="-";
string         comment_str2="-";
int		      DebugLevel = 2;


double 	ExtMapBuffer0[];	// Contains (minutes until) each news event
double 	ExtMapBuffer1[];	// Contains only most recent and next news event ([0] & [1])
double 	ExtMapBuffer2[];	// Contains impact value for most recent and next news event
double 	ExtMapBuffer3[];  // newsIdx
double 	ExtMapBuffer4[];  // NewsDateTime
double 	ExtMapBuffer5[];  // Impact
double 	ExtMapBuffer6[];  // Previous
double 	ExtMapBuffer7[];  // Forecast

//fxfishermann -->http://64.22.88.90/forexcalendar/forecast.aspx
string	    sUrl = "http://www.forexfactory.com/weekly_calendar.xml";
int 	       xmlHandle;
int 	       logHandle = -1;
int 	       BoEvent, finalend, end, i;
int		    begin;
string 	    mainData[100][7];
int 	       minsTillNews;
string 	    sData, csvoutput;
string	    commentStr;
int		    tmpMins;
int		    idxOfNext;
int		    dispMinutes[2];
string 	    dispTitle[2], 
		       dispCountry[2], 
		       dispImpact[2], 
		       dispForecast[2], 
		       dispPrevious[2];
		       
string	    dispSeparator1 = "===================";

static bool  FirstRun = true;
static int	 PrevMinute = -1;


int init()
{

   create_globals();
   
	Print("In Init()...\n");
	
	SetIndexStyle(0, DRAW_NONE);
	SetIndexBuffer(0, ExtMapBuffer0);
   SetIndexEmptyValue(0,0.0);
	SetIndexStyle(1, DRAW_NONE);
	SetIndexBuffer(1, ExtMapBuffer1);
   SetIndexEmptyValue(1,0.0);
	SetIndexStyle(2, DRAW_NONE);
	SetIndexBuffer(2, ExtMapBuffer2);
   SetIndexEmptyValue(2,0.0);
	SetIndexStyle(3, DRAW_NONE);
	SetIndexBuffer(3, ExtMapBuffer3);
   SetIndexEmptyValue(3,0.0);	
   
	SetIndexStyle(4, DRAW_NONE);
	SetIndexBuffer(4, ExtMapBuffer4);
   SetIndexEmptyValue(4,0.0);	
	SetIndexStyle(5, DRAW_NONE);
	SetIndexBuffer(5, ExtMapBuffer5);
   SetIndexEmptyValue(5,0.0);	
	SetIndexStyle(6, DRAW_NONE);
	SetIndexBuffer(6, ExtMapBuffer6);
   SetIndexEmptyValue(6,0.0);	
	SetIndexStyle(7, DRAW_NONE);
	SetIndexBuffer(7, ExtMapBuffer7);
   SetIndexEmptyValue(7,0.0);	

	
   SetIndexEmptyValue(3,0.0);
	IndicatorShortName("FFCal");
	SetIndexLabel(0, "MinsBeforeNews");
	SetIndexLabel(1, "RecentandNext"); 
	SetIndexLabel(2, "ImpactRecentandNext"); 
   SetIndexLabel(3, "newsIdx"); 	
   SetIndexLabel(4, "NewsDateTime"); 	
   SetIndexLabel(5, "Impact"); 	
   SetIndexLabel(6, "Previous"); 	
   SetIndexLabel(7, "Forecast"); 	            

	return(0);
}


int deinit()
{
  
   deinit_globals();
   
	ObjectDelete("Sponsor"); 
	ObjectDelete("Minutes"); 
	ObjectDelete("Impact");
	ObjectDelete("Previous"); 
	ObjectDelete("Forecast");
	ObjectDelete("Separator1"); 
	ObjectDelete("Separator2");

	ObjectDelete("Sponsor2"); 
	ObjectDelete("Minutes2"); 
	ObjectDelete("Impact2");
	ObjectDelete("Previous2"); 
	ObjectDelete("Forecast2");

	return(0);
}


int start()
{
	int 		newsIdx = 0;
	int 		nextNewsIdx = -1;
	int 		next;
	string 	myEvent;
	bool 		skip;
	datetime newsTime;
	
	init_global1();
	init_global2();
	
	// If we are not logging, then do not output debug statements either
	if (!EnableLogging)
		DebugLevel = 0;
				
	commentStr = "Forex Factory Calendar";
	string xmlFileName = Month() + "-" + Day() + "-" + Year() + "-" + "FFCal.xml";
	Print("STARTED");
	
	// Refresh FF calendar on startup, and once at the top of each hour
	if (FirstRun || Minute() == 0)
	{
		FirstRun = false;

		if (DebugLevel > 1)
			Print("sUrl == ", sUrl);
		
		if (DebugLevel > 0)
			Print("Grabbing Web, url = ", sUrl);
	
		// THIS CALL WAS DONATED BY PAUL TO HELP FIX THE RESOURCE ERROR
		GrabWeb(sUrl, sData);

		if (DebugLevel > 0)
		{
			Print("Opening XML file...\n");
			Print(sData);
		}

		// Write the contents of the ForexFactory page to an .htm file
		xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_WRITE);
		if (xmlHandle < 1)
		{
			if (DebugLevel > 0)
				Print("Can\'t open new xml file, the last error is ", GetLastError());
			return(false);
		}
		FileWriteString(xmlHandle, sData, StringLen(sData));
		FileClose(xmlHandle);

		if (DebugLevel > 0)
			Print("Wrote XML file...\n");

		// THIS BLOCK OF CODE DONATED BY WALLY TO FIX THE RESOURCE ERROR
		//--- Look for the end XML tag to ensure that a complete page was downloaded ---//
		end = StringFind(sData, "</weeklyevents>", 0);

		if (end <= 0)
		{
			Alert("GetNewsFF Error - Web page download was not complete!");
			return(false);
		}
		//-------------------------------------------------------------------------------//
	}

	// = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
	// Perform remaining checks once per minute
	if (Minute() == PrevMinute)
		return (true);
	PrevMinute = Minute();
	Print("NEW MINUTE...");

	// Open the log file (will not open if logging is turned off)
	OpenLog("FFCal");

	// Init the buffer array to zero just in case
	ArrayInitialize(ExtMapBuffer0, 0);
	ArrayInitialize(ExtMapBuffer1, 0);
	ArrayInitialize(ExtMapBuffer2, 0);
	ArrayInitialize(ExtMapBuffer3, 0);
	ArrayInitialize(ExtMapBuffer4, 0);
	ArrayInitialize(ExtMapBuffer5, 0);
	ArrayInitialize(ExtMapBuffer6, 0);
	ArrayInitialize(ExtMapBuffer7, 0);				
	
	// Open the XML file
	xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ);
	if (xmlHandle < 0)
	{
		Print("Can\'t open xml file: ", xmlFileName, ".  The last error is ", GetLastError());
		return(false);
	}
	if (DebugLevel > 0)
		Print("XML file open must be okay");
	
	// Read in the whole XML file
	// Ave file length == ~7K, so 65536 should always read whole file
	sData = FileReadString(xmlHandle, 65536);	
	FileClose(xmlHandle);

	// Get the currency pair, and split it into the two countries
	string pair = Symbol();
	string cntry1 = StringSubstr(pair, 0, 3);
	string cntry2 = StringSubstr(pair, 3, 3);
	if (DebugLevel > 0)
		Print("cntry1 = ", cntry1, "    cntry2 = ", cntry2);
	
	if (DebugLevel > 0)
		Log("Weekly calendar for " + pair + "\n\n");

	// -------------------------------------------------
	// Parse the XML file looking for an event to report
	// -------------------------------------------------
	
	tmpMins = 10080;	// (a week)
	BoEvent = 0;

	string sTags[7] = { "<title>", "<country>", "<date>", "<time>", "<impact>", "<forecast>", "<previous>" };
	string eTags[7] = { "</title>", "</country>", "</date>", "</time>", "</impact>", "</forecast>", "</previous>" };


   ////////////////////////////////////////////////////////////////////////////////////////////////1
	while (true)
	{
		BoEvent = StringFind(sData, "<event>", BoEvent);
		if (BoEvent == -1)
			break;
			
		BoEvent += 7;	
		next = StringFind(sData, "</event>", BoEvent);
		myEvent = StringSubstr(sData, BoEvent, next - BoEvent);
		BoEvent = next;
		
		begin = 0;
		skip = false;
		for (i=0; i < 7; i++)
		{
			mainData[newsIdx][i] = "";
			next = StringFind(myEvent, sTags[i], begin);
			
			// Within this event, if tag not found, then it must be missing; skip it
			if (next == -1)
				continue;
			else
			{
				// We must have found the sTag okay...
				begin = next + StringLen(sTags[i]);			// Advance past the start tag
				end = StringFind(myEvent, eTags[i], begin);	// Find start of end tag
				if (end > begin && end != -1)
				{
					// Get data between start and end tag
					mainData[newsIdx][i] = StringSubstr(myEvent, begin, end - begin);
				}
			}
		}
		
      //		for (i=6; i >= 0; i--)
      //			Print(sTags[i], "  =  ", mainData[newsIdx][i]);

		// = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =
		// Test against filters that define whether we want to 
		// skip this particular annoucemnt
		if (cntry1 != mainData[newsIdx][COUNTRY] && cntry2 != mainData[newsIdx][COUNTRY])
			skip = true;
		if (!IncludeHigh && mainData[newsIdx][IMPACT] == "High") 
			skip = true;
		if (!IncludeMedium && mainData[newsIdx][IMPACT] == "Medium") 
			skip = true;
		if (!IncludeLow && mainData[newsIdx][IMPACT] == "Low") 
			skip = true;
		if (!IncludeSpeaks && (StringFind(mainData[newsIdx][TITLE], "speaks") != -1 || 
								StringFind(mainData[newsIdx][TITLE], "Speaks") != -1) ) 
			skip = true;
		if (mainData[newsIdx][TIME] == "All Day" || 
			mainData[newsIdx][TIME] == "Tentative" ||
			mainData[newsIdx][TIME] == "") 
			skip = true;
		// = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =   = - =


		// If not skipping this event, then log it into the draw buffers
		if (!skip)
		{
			// If we got this far then we need to calc the minutes until this event
			//
			// First, convert the announcement time to seconds, GMT
			newsTime = StrToTime(MakeDateTime(mainData[newsIdx][DATE], mainData[newsIdx][TIME]));
			
			// Now calculate the minutes until this announcement (may be negative)
			minsTillNews = (newsTime - TimeGMT()) / 60;

			// Keep track of the most recent news announcement.
			// Do that by saving each one until we get to the 
			// first annoucement that isn't in the past; i.e.
			// minsTillNews > 0.  Then, keep this one instead for
			// display, but only once the minutes until the next 
			// news is SMALLER than the minutes since the last.
//			Print("Mins till event: ", minsTillNews);
			if (minsTillNews < 0 || MathAbs(tmpMins) > minsTillNews)
			{
				idxOfNext = newsIdx;
				tmpMins	= minsTillNews;
			}
			
			Log("Weekly calendar for " + pair + "\n\n");
			if (DebugLevel > 0)
			{
				Log("FOREX FACTORY\nTitle: "  + mainData[newsIdx][TITLE] + 
									"\nCountry: "  + mainData[newsIdx][COUNTRY] + 
									"\nDate: "     + mainData[newsIdx][DATE] + 
									"\nTime: "     + mainData[newsIdx][TIME] + 
									"\nImpact: "   + mainData[newsIdx][IMPACT] + 
									"\nForecast: " + mainData[newsIdx][FORECAST] + 
									"\nPrevious: " + mainData[newsIdx][PREVIOUS] + "\n\n");
			}
			
			// Do alert if user has enabled
			string strNextNews=mainData[newsIdx][TITLE];
			datetime NewsTimeServer=newsTime;//+TimeZoneServer()*3600			
			if (Alert1MinsBefore != -1 && minsTillNews <= Alert1MinsBefore)
			{
				//Alert(Alert1MinsBefore, " minutes until news for ", pair, ": ", mainData[newsIdx][TITLE]);
            ObjectCreate("_"+strNextNews, OBJ_TEXT, 0, NewsTimeServer, Close[0]);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==3)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Red);          
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==2)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Orange);          
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==1)
              ObjectSet("_"+strNextNews, OBJPROP_COLOR, Yellow);          
            ObjectSetText("_"+strNextNews, strNextNews, 8);          
            ObjectSet("_"+strNextNews, OBJPROP_ANGLE, 90);                                        		
     		   ObjectCreate(strNextNews, OBJ_VLINE, 0, NewsTimeServer,Close[0]);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==3)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Red);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==2)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Orange);
            if(ImpactToNumber(mainData[newsIdx][IMPACT])==1)     		   
     		     ObjectSet(strNextNews, OBJPROP_COLOR, Yellow);
     		   ObjectSet(strNextNews, OBJPROP_STYLE, STYLE_DOT);
            //ObjectSet(strNextNews, OBJPROP_BACK, true);     

			}	
				
			// Buffers are set up as so: 
			// ExtMapBuffer0 contains the time UNTIL each announcement (can be negative)
			// e.g. [0] = -372; [1] = 25; [2] = 450; [3] = 1768 (etc.)
			// ExtMapBuffer1[0] has the mintutes since the last annoucement.
			// ExtMapBuffer1[1] has the mintutes until the next annoucement.
			comment_str1="Server Time Zone("+StrTimeZoneServer()+")\n"+
			             "Local Time Zone("+StrTimeZoneLocal()+")\n"+
			             "GMT Time:"+TimeToStr(TimeGMT(),TIME_DATE|TIME_MINUTES)+"\n"+
			             "Server Time:"+TimeToStr(TimeGMT()+TimeZoneServer()*3600,TIME_DATE|TIME_MINUTES)+"\n"+
			             "Local Time:"+TimeToStr(TimeGMT()+TimeZoneLocal()*3600,TIME_DATE|TIME_MINUTES);
			  
         comment_str2="";//"Server Time Zone("+StrTimeZoneServer()+")\n"+
			             //"Local Time Zone("+StrTimeZoneLocal()+")\n"+
			             //"GMT Time:"+TimeToStr(TimeGMT(),TIME_DATE|TIME_MINUTES);
			   			         
         //  comment_str1=CleanStr("-23.5");

			
			ExtMapBuffer0[newsIdx] = minsTillNews;
			ExtMapBuffer3[newsIdx] = newsIdx;  // newsIdx
         ExtMapBuffer4[newsIdx] = newsTime;  // NewsDateTime,GMT		
         ExtMapBuffer5[newsIdx] = ImpactToNumber(mainData[newsIdx][IMPACT]);  // Impact
         ExtMapBuffer6[newsIdx] = PreviousToNumber(mainData[newsIdx][PREVIOUS]);//
         ExtMapBuffer7[newsIdx] = ForecastToNumber(mainData[newsIdx][FORECAST]);//
			newsIdx++;
		}
	}
   ////////////////////////////////////////////////////////////////////////////////////////////
	// Cycle through the events array and pick out the most recent 
	// past and the next coming event to put into ExtMapBuffer1. 
	// Put the corresponding impact for these two into ExtMapBuffer2.
	bool first = true;
	ExtMapBuffer1[0] = 0;
	ExtMapBuffer1[1] = 0;
	ExtMapBuffer2[0] = 0;
	ExtMapBuffer2[1] = 0;
	string outNews = "Minutes until news events for " + pair + " : ";
	for (i=0; i < newsIdx; i++)	
	{
		outNews = outNews + ExtMapBuffer0[i] + ", ";
		if (ExtMapBuffer0[i] > 0 && first)
		{
			first = false;
			
			// Put the relevant info into the indicator buffers...
			if (i > 0)
			{
				ExtMapBuffer1[0] = ExtMapBuffer0[i-1];
				ExtMapBuffer2[0] = ImpactToNumber(mainData[i-1][IMPACT]);
			}
			ExtMapBuffer1[1] = ExtMapBuffer0[i];
			ExtMapBuffer2[1] = ImpactToNumber(mainData[i][IMPACT]);
		}
		
		// Also use this loop to set which information to display
		if (i == idxOfNext)////////////////////////////////////////
		{
			dispTitle[0]	= mainData[i][TITLE];
			dispCountry[0] 	= mainData[i][COUNTRY];
			dispImpact[0] 	= mainData[i][IMPACT];
			dispForecast[0] = mainData[i][FORECAST];
			dispPrevious[0] = mainData[i][PREVIOUS];
			dispMinutes[0] 	= ExtMapBuffer0[i];
		}
		if (i == idxOfNext + 1)////////////////////////////////////
		{
			dispTitle[1]	= mainData[i][TITLE];
			dispCountry[1] 	= mainData[i][COUNTRY];
			dispImpact[1] 	= mainData[i][IMPACT];
			dispForecast[1] = mainData[i][FORECAST];
			dispPrevious[1] = mainData[i][PREVIOUS];
			dispMinutes[1] 	= ExtMapBuffer0[i];
		}
	}	
	// If we are past all news events, then neither one will have been 
	// set, so set the past event to the last (negative) minutes
	if (ExtMapBuffer1[0] == 0 && ExtMapBuffer1[1] == 0)
		ExtMapBuffer1[0] = ExtMapBuffer0[i-1];

	
	// For debugging...Print the tines until news events, as a "Comment"
	if (DebugLevel > 0)
	{
		Print(outNews);
		Print("LastMins (ExtMapBuffer1[0]) = ", ExtMapBuffer1[0]);
		Print("NextMins (ExtMapBuffer1[1]) = ", ExtMapBuffer1[1]);
	}
	 
    if (TextOutput)
      if(Text)
        OutputToChart_text();
      else
        Comment("");  
    else  
      if(Text)
        OutputToChart();

	if (logHandle != -1)
		FileClose(logHandle);
	return (0);
}


void OutputToChart()
{
	// Added by Robert for using TxtSize and TxtColor for easier reading
	int curY = 10;
	ObjectCreate("Separator1", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Separator1", dispSeparator1, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Separator1", OBJPROP_CORNER, 0);
	ObjectSet("Separator1", OBJPROP_XDISTANCE, 10);
	ObjectSet("Separator1", OBJPROP_YDISTANCE, curY); 

	curY = curY + TxtSize + 4;
	ObjectCreate("Sponsor", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Sponsor", commentStr, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Sponsor", OBJPROP_CORNER, 0);
	ObjectSet("Sponsor", OBJPROP_XDISTANCE, 10);
	ObjectSet("Sponsor", OBJPROP_YDISTANCE, curY);       

	curY = curY + TxtSize + 4;
	ObjectCreate("Separator2", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Separator2", dispSeparator1, TxtSize, "Arial Bold", TxtColorTitle);
	ObjectSet("Separator2", OBJPROP_CORNER, 0);
	ObjectSet("Separator2", OBJPROP_XDISTANCE, 10);
	ObjectSet("Separator2", OBJPROP_YDISTANCE, curY);   

	curY = curY + TxtSize + 4;
	ObjectCreate("Minutes", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Minutes", dispMinutes[0] + " Mins until " + dispCountry[0] + ": " + dispTitle[0], TxtSize, "Arial Bold", TxtColorNews);
	ObjectSet("Minutes", OBJPROP_CORNER, 0);
	ObjectSet("Minutes", OBJPROP_XDISTANCE, 10);
	ObjectSet("Minutes", OBJPROP_YDISTANCE, curY);

	curY = curY + TxtSize + 4;
	ObjectCreate("Impact", OBJ_LABEL, 0, 0, 0);
	ObjectSetText("Impact", "Impact: " + dispImpact[0], TxtSize, "Arial Bold", TxtColorImpact);
	ObjectSet("Impact", OBJPROP_CORNER, 0);
	ObjectSet("Impact", OBJPROP_XDISTANCE, 10);
	ObjectSet("Impact", OBJPROP_YDISTANCE, curY);

	if (dispPrevious[0] != "")
	{
		curY = curY + TxtSize + 4;
		ObjectCreate("Previous", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Previous", "Previous: " + dispPrevious[0], TxtSize, "Arial Bold", TxtColorPrevious);
		ObjectSet("Previous", OBJPROP_CORNER, 0);
		ObjectSet("Previous", OBJPROP_XDISTANCE, 10);
		ObjectSet("Previous", OBJPROP_YDISTANCE, curY);
	}
	
	if (dispForecast[0] != "")
	{
		curY = curY + TxtSize + 4;
		ObjectCreate("Forecast", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Forecast", "Forecast: " + dispForecast[0], TxtSize, "Arial Bold", TxtColorForecast);
		ObjectSet("Forecast", OBJPROP_CORNER, 0);
		ObjectSet("Forecast", OBJPROP_XDISTANCE, 10);
		ObjectSet("Forecast", OBJPROP_YDISTANCE, curY); 
	}
	
	// Do second news event
	if (ShowNextTwoEvents)
	{
		curY = curY + TxtSize + 20;
		ObjectCreate("Minutes2", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Minutes2", dispMinutes[1] + " Mins until " + dispCountry[1] + ": " + dispTitle[1], TxtSize, "Arial Bold", TxtColorNews);
		ObjectSet("Minutes2", OBJPROP_CORNER, 0);
		ObjectSet("Minutes2", OBJPROP_XDISTANCE, 10);
		ObjectSet("Minutes2", OBJPROP_YDISTANCE, curY);

		curY = curY + TxtSize + 4;
		ObjectCreate("Impact2", OBJ_LABEL, 0, 0, 0);
		ObjectSetText("Impact2", "Impact: " + dispImpact[1], TxtSize, "Arial Bold", TxtColorImpact);
		ObjectSet("Impact2", OBJPROP_CORNER, 0);
		ObjectSet("Impact2", OBJPROP_XDISTANCE, 10);
		ObjectSet("Impact2", OBJPROP_YDISTANCE, curY);

		if (dispPrevious[1] != "")
		{
			curY = curY + TxtSize + 4;
			ObjectCreate("Previous2", OBJ_LABEL, 0, 0, 0);
			ObjectSetText("Previous2", "Previous: " + dispPrevious[1], TxtSize, "Arial Bold", TxtColorPrevious);
			ObjectSet("Previous2", OBJPROP_CORNER, 0);
			ObjectSet("Previous2", OBJPROP_XDISTANCE, 10);
			ObjectSet("Previous2", OBJPROP_YDISTANCE, curY);
		}
		
		if (dispForecast[1] != "")
		{
			curY = curY + TxtSize + 4;
			ObjectCreate("Forecast2", OBJ_LABEL, 0, 0, 0);
			ObjectSetText("Forecast2", "Forecast: " + dispForecast[1], TxtSize, "Arial Bold", TxtColorForecast);
			ObjectSet("Forecast2", OBJPROP_CORNER, 0);
			ObjectSet("Forecast2", OBJPROP_XDISTANCE, 10);
			ObjectSet("Forecast2", OBJPROP_YDISTANCE, curY); 
		}
	}
}


void OutputToChart_text()
{

	ObjectDelete("Sponsor"); 
	ObjectDelete("Minutes"); 
	ObjectDelete("Impact");
	ObjectDelete("Previous"); 
	ObjectDelete("Forecast");
	ObjectDelete("Separator1"); 
	ObjectDelete("Separator2");

	ObjectDelete("Sponsor2"); 
	ObjectDelete("Minutes2"); 
	ObjectDelete("Impact2");
	ObjectDelete("Previous2"); 
	ObjectDelete("Forecast2");

   string str1=comment_str1+"\n"+dispSeparator1
               +"\n"+commentStr
               +"\n"+dispSeparator1
               +"\n"+dispMinutes[0] + " Mins until " + dispCountry[0] + ": " + dispTitle[0]
               +"\n"+"Impact: " + dispImpact[0];

	if (dispPrevious[0] != "")
	{
		str1=str1+"\n"+"Previous: " + dispPrevious[0];
   }
	
	if (dispForecast[0] != "")
	{
		str1=str1+"\n"+"Forecast: " + dispForecast[0];
	}

	// Do second news event
	if (ShowNextTwoEvents)
	{
		str1=str1+"\n"+dispMinutes[1] + " Mins until " + dispCountry[1] + ": " + dispTitle[1]

		+"\n"+"Impact: " + dispImpact[1];

	   if (dispPrevious[1] != "")
 	   {
			str1=str1+"\n"+"Previous: " + dispPrevious[1];
		}
		
		if (dispForecast[1] != "")
		{
			str1=str1+"\n"+"Forecast: " + dispForecast[1];
		}
	}
	str1=str1+"\n"+dispSeparator1+"\n"+comment_str2;
	//Comment("");
	Comment(str1);
}



int ImpactToNumber(string impact)
{
	if (impact == "High")
		return (3);
	if (impact == "Medium")
		return (2);
	if (impact == "Low")
		return (1);
	else
		return (0);
}

double PreviousToNumber(string Previous)
{
  return(StrToDouble(CleanStr(Previous)));  // Previous);  // Forecast);
}

double ForecastToNumber(string Forecast)
{
  return(StrToDouble(CleanStr(Forecast)));  // Forecast);
}
/*
ascii[56, 57, 58, 59] = '89:;'
ascii[52, 53, 54, 55] = '4567'
ascii[48, 49, 50, 51] = '0123'
ascii[44, 45, 46, 47] = ',-./'
*/
string CleanStr(string str)
{
	// change illegal filename characters in str to '-'
	int i;
   string r1;
   
   for (i=0;i<StringLen(str);i++)
   {
     if((58>StringGetChar(str,i)) && (47<StringGetChar(str,i)))
       r1=r1+CharToStr(StringGetChar(str,i));
     if((CharToStr(StringGetChar(str,i))=="."))  
       r1=r1+CharToStr(StringGetChar(str,i));
     if((CharToStr(StringGetChar(str,i))=="-"))  	 
       r1=r1+CharToStr(StringGetChar(str,i));
   }
	return(r1);
}

   
string MakeDateTime(string strDate, string strTime)
{
	// converts forexfactory time & date into yyyy.mm.dd hh:mm
	int n1stDash = StringFind(strDate, "-");
	int n2ndDash = StringFind(strDate, "-", n1stDash+1);

	string strMonth = StringSubstr(strDate, 0, 2);
	string strDay = StringSubstr(strDate, 3, 2);
	string strYear = StringSubstr(strDate, 6, 2); 
	strYear = "20" + strYear;
	
	int nTimeColonPos	= StringFind(strTime, ":");
	string strHour = StringSubstr(strTime, 0, nTimeColonPos);
	string strMinute = StringSubstr(strTime, nTimeColonPos+1, 2);
	string strAM_PM = StringSubstr(strTime, StringLen(strTime)-2);

	int nHour24 = StrToInteger(strHour);
	if (strAM_PM == "pm" || strAM_PM == "PM" && nHour24 != 12)
	{
		nHour24 += 12;
	}
	string strHourPad = "";
	if (nHour24 < 10) 
		strHourPad = "0";

	return(StringConcatenate(strYear, ".", strMonth, ".", strDay, " ", strHourPad, nHour24, ":", strMinute));
}


//=================================================================================================
//=================================================================================================
//====================================   GrabWeb Functions   ======================================
//=================================================================================================
//=================================================================================================
// Main Webscraping function
// ~~~~~~~~~~~~~~~~~~~~~~~~~
// bool GrabWeb(string strUrl, string& strWebPage)
// returns the text of any webpage. Returns false on timeout or other error
// 
// Parsing functions
// ~~~~~~~~~~~~~~~~~
// string GetData(string strWebPage, int nStart, string strLeftTag, string strRightTag, int& nPos)
// obtains the text between two tags found after nStart, and sets nPos to the end of the second tag
//
// void Goto(string strWebPage, int nStart, string strTag, int& nPos)
// Sets nPos to the end of the first tag found after nStart 

bool bWinInetDebug = false;

int hSession_IEType;
int hSession_Direct;
int Internet_Open_Type_Preconfig = 0;
int Internet_Open_Type_Direct = 1;
int Internet_Open_Type_Proxy = 3;
int Buffer_LEN = 13;

#import "wininet.dll"

#define INTERNET_FLAG_PRAGMA_NOCACHE    0x00000100 // Forces the request to be resolved by the origin server, even if a cached copy exists on the proxy.
#define INTERNET_FLAG_NO_CACHE_WRITE    0x04000000 // Does not add the returned entity to the cache. 
#define INTERNET_FLAG_RELOAD            0x80000000 // Forces a download of the requested file, object, or directory listing from the origin server, not from the cache.

int InternetOpenA(
	string 	sAgent,
	int		lAccessType,
	string 	sProxyName="",
	string 	sProxyBypass="",
	int 	lFlags=0
);

int InternetOpenUrlA(
	int 	hInternetSession,
	string 	sUrl, 
	string 	sHeaders="",
	int 	lHeadersLength=0,
	int 	lFlags=0,
	int 	lContext=0 
);

int InternetReadFile(
	int 	hFile,
	string 	sBuffer,
	int 	lNumBytesToRead,
	int& 	lNumberOfBytesRead[]
);

int InternetCloseHandle(
	int 	hInet
);
#import


int hSession(bool Direct)
{
	string InternetAgent;
	if (hSession_IEType == 0)
	{
		InternetAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Q312461)";
		hSession_IEType = InternetOpenA(InternetAgent, Internet_Open_Type_Preconfig, "0", "0", 0);
		hSession_Direct = InternetOpenA(InternetAgent, Internet_Open_Type_Direct, "0", "0", 0);
	}
	if (Direct) 
	{ 
		return(hSession_Direct); 
	}
	else 
	{
		return(hSession_IEType); 
	}
}


bool GrabWeb(string strUrl, string& strWebPage)
{
	int 	hInternet;
	int		iResult;
	int 	lReturn[]={1};
	string 	sBuffer="x";
	int 	bytes;
	
	hInternet = InternetOpenUrlA(hSession(FALSE), strUrl, "0", 0, 
								INTERNET_FLAG_NO_CACHE_WRITE | 
								INTERNET_FLAG_PRAGMA_NOCACHE | 
								INTERNET_FLAG_RELOAD, 0);
								
	if (bWinInetDebug) 
		Log("hInternet: " + hInternet);   
	if (hInternet == 0) 
		return(false);
		
	iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
	
	if (bWinInetDebug) 
		Log("iResult: " + iResult);
	if (bWinInetDebug) 
		Log("lReturn: " + lReturn[0]);
	if (bWinInetDebug) 
		Log("iResult: " + iResult);
	if (bWinInetDebug) 
		Log("sBuffer: " +  sBuffer);
	if (iResult == 0) 
		return(false);
	bytes = lReturn[0];

	strWebPage = StringSubstr(sBuffer, 0, lReturn[0]);
	
	// If there's more data then keep reading it into the buffer
	while (lReturn[0] != 0)
	{
		iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
		if (lReturn[0]==0) 
			break;
		bytes = bytes + lReturn[0];
		strWebPage = strWebPage + StringSubstr(sBuffer, 0, lReturn[0]);
	}

	iResult = InternetCloseHandle(hInternet);
	if (iResult == 0) 
		return(false);
	if (bWinInetDebug) 
	{ 	
		Log("iResult: " + iResult);
		int handle = FileOpen("Grabweb.htm",FILE_BIN|FILE_READ|FILE_WRITE);
		FileWriteString(handle,strWebPage,StringLen(strWebPage));
		FileClose(handle);
	}
	return(true);
}


/*
string GetData(string strWebPage, int nStart, string strLeftTag, string strRightTag, int& nPos)
{
	nPos = StringFind(strWebPage,strLeftTag,nStart);
	if (nPos == -1)
	{
		if (bWinInetDebug) Log("GetData(WebPage,"+nStart+","+strLeftTag+","+strRightTag+"): not found, returning empty string and -1");
		return("");
	}
	nPos += StringLen(strLeftTag);
	int nEnd = StringFind(strWebPage,strRightTag,nPos+1);
	string strData = StringTrimLeft(StringTrimRight(StringSubstr(strWebPage,nPos,nEnd-nPos)));
	if (bWinInetDebug) Log("GetData(WebPage,"+nStart+","+strLeftTag+","+strRightTag+") returning "+strData+" and "+nPos);
	return(strData);
}

void Goto(string strWebPage, int nStart, string strTag, int& nPos)
{
	nPos = StringFind(strWebPage,strTag,nStart);
	if (nPos == -1) 
	{
		if (bWinInetDebug) Log("Goto("+nStart+","+strTag+") not found. Returning -1");
		return;
	}
	nPos += StringLen(strTag);
	if (bWinInetDebug) Log("Goto("+nStart+","+strTag+") returning "+nPos);
}

void SaveWebPage(string strWebPage, string strID)
{
	string strFilename = strID+"_"+Year()+Pad(Month())+Pad(Day())+"-"+Pad(Hour())+Pad(Minute())+Pad(Seconds())+".htm";
	int handle = FileOpen(strFilename,FILE_BIN|FILE_WRITE);
	FileWriteString(handle,strWebPage,StringLen(strWebPage));
   FileClose(handle);
}


string Pad(int n)
{
	string strPad = "";
	if (n <= 9) strPad = "0";
	return(strPad+n);
}
*/

//=================================================================================================
//=================================================================================================
//===================================   LogUtils Functions   ======================================
//=================================================================================================
//=================================================================================================

void OpenLog(string strName)
{
	if (!EnableLogging) 
		return;

	if (logHandle <= 0)
	{
		string strMonthPad = "";
 	 	string strDayPad = "";
  		if (Month() < 10) 
  			strMonthPad = "0";
  		if (Day() < 10) 
  			strDayPad = "0";
  			
  		string strFilename = StringConcatenate(strName, "_", Year(), strMonthPad, Month(), strDayPad, Day(), "_log.txt");
  		
		logHandle = FileOpen(strFilename,FILE_CSV|FILE_READ|FILE_WRITE);
		Print("logHandle =================================== ", logHandle);
	}
	if (logHandle > 0)
	{
		FileFlush(logHandle);
		FileSeek(logHandle, 0, SEEK_END);
	}
}


void Log(string msg)
{
	if (!EnableLogging) 
		return;
		
	if (logHandle <= 0) 
		return;
		
	msg = TimeToStr(TimeCurrent(),TIME_DATE|TIME_MINUTES|TIME_SECONDS) + " " + msg;
	FileWrite(logHandle,msg);
}

//=================================================================================================
//=================================================================================================
//===================================   Timezone Functions   ======================================
//=================================================================================================
//=================================================================================================


#import "kernel32.dll"
int  GetTimeZoneInformation(int& TZInfoArray[]);
#import

#define TIME_ZONE_ID_UNKNOWN   0
#define TIME_ZONE_ID_STANDARD  1
#define TIME_ZONE_ID_DAYLIGHT  2

// Local timezone in hours, adjusting for daylight saving
double TimeZoneLocal()
{
	int TZInfoArray[43];

	switch(GetTimeZoneInformation(TZInfoArray))
	{
	case TIME_ZONE_ID_UNKNOWN: 
		Print("Error obtaining PC timezone from GetTimeZoneInformation in kernel32.dll. Returning 0");
		return(0);

	case TIME_ZONE_ID_STANDARD:
		return(TZInfoArray[0]/(-60.0));
	
	case TIME_ZONE_ID_DAYLIGHT:
		return((TZInfoArray[0]+TZInfoArray[42])/(-60.0));
		
	default:
		Print("Unkown return value from GetTimeZoneInformation in kernel32.dll. Returning 0");
		return(0);
	}
}

string StrTimeZoneLocal()
{
  string r1;
  if(TimeZoneLocal()>=0)
    r1="GMT+"+DoubleToStr(TimeZoneLocal(),0);
  else
    r1="GMT"+DoubleToStr(TimeZoneLocal(),0);    
  return(r1);   
}



// Server timezone in hours
double TimeZoneServer()
{
	int ServerToLocalDiffMinutes = (TimeCurrent()-TimeLocal())/60;
	
	// round to nearest 30 minutes to allow for inaccurate PC clock
	int nHalfHourDiff = MathRound(ServerToLocalDiffMinutes/30.0);
	ServerToLocalDiffMinutes = nHalfHourDiff*30;
	return(TimeZoneLocal() + ServerToLocalDiffMinutes/60.0);
}

string StrTimeZoneServer()
{
  string r1;
  if(TimeZoneServer()>=0)
    r1="GMT+"+DoubleToStr(TimeZoneServer(),0);
  else
    r1="GMT"+DoubleToStr(TimeZoneServer(),0);    
  return(r1);   
}

// Uses local PC time, local PC timezone, and server time to calculate GMT time at arrival of last tick
datetime TimeGMT()
{
	// two ways of calculating
	// 1. From PC time, which may not be accurate
	// 2. From server time. Most accurate except when server is down on weekend
	datetime dtGmtFromLocal = TimeLocal() - TimeZoneLocal()*3600;
	datetime dtGmtFromServer = TimeCurrent() - TimeZoneServer()*3600;

	// return local-derived value if server value is out by more than 5 minutes, eg during weekend
	if (dtGmtFromLocal > dtGmtFromServer + 300)
	{
		return(dtGmtFromLocal);
	}
	else
	{
		return(dtGmtFromServer);
	}	
}

//=================================================================================================
//=================================================================================================
//=================================   END IMPORTED FUNCTIONS  =====================================
//=================================================================================================
//=================================================================================================

void create_globals()
{

   string Global1 = /*Symbol() + */"_GMTdiffServer";
   if (! GlobalVariableCheck(Global1))
     GlobalVariableSet(Global1,TimeZoneServer()); //Global Symbol Comment
   string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
   if (! GlobalVariableCheck(Global2))
     GlobalVariableSet(Global2,TimeZoneLocal()); //Global Symbol Comment
}


void deinit_globals()
{

   string Global1 = /*Symbol() + */"_GMTdiffServer";
   if (GlobalVariableCheck(Global1))
     GlobalVariableDel(Global1); //Global Symbol Comment
   string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
   if (GlobalVariableCheck(Global2))
     GlobalVariableDel(Global2); //Global Symbol Comment
    
}

//////////////////////////////////////////////////////

void init_global1() //100,7
{
         string Global1 = /*Symbol() + */"_GMTdiffServer";
         if (GlobalVariableCheck(Global1))
           GlobalVariableSet(Global1,TimeZoneServer());

}

void init_global2() //100,7
{
         string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
         if (GlobalVariableCheck(Global2))
           GlobalVariableSet(Global2,TimeZoneLocal());

}

double get_global1()
{
        double r1;
        string Global1 = /*Symbol() + */"_GMTdiffServer";
        if (GlobalVariableCheck(Global1))
          r1=GlobalVariableGet(Global1);
        else
          r1=0;  
        return(r1);
}

double get_global2()
{
         double r2;
         string Global2 = /*Symbol() +*/ "_GMTdiffLocal";
         if (GlobalVariableCheck(Global2))
           r2=GlobalVariableGet(Global2);
         else
           r2=0;  
         return(r2);
}


double news_FF_code(string Curr1,string News1)
{
  double c1,n1,r1;
  
  c1=curr_FF_code(Curr1);
  n1=news1_FF_code(News1);
  r1=c1+n1;
  
  return(r1);

}

double curr_FF_code(string Curr1)
{
  double r1=0;//ALL

if (Curr1=="EUR")
  r1=1000;
else if (Curr1=="USD") 
  r1=2000;
else if (Curr1=="JPY") 
  r1=3000;
else if (Curr1=="GBP") 
  r1=4000;
else if (Curr1=="AUD") 
  r1=5000;
else if (Curr1=="CAD") 
  r1=6000;
else if (Curr1=="CHF") 
  r1=7000;
else if (Curr1=="NZD") 
  r1=8000;
  
  return (r1);
}

double news1_FF_code(string News1)
{
  double r1=0;
if (News1=="ADP Nonfarm Employment Change")
  r1=1;
else if (News1=="All Industries Activity Index m/m") 
  r1=2;
else if (News1=="All Industries Index m/m") 
  r1=3;
else if (News1=="Average Cash Earnings y/y") 
  r1=4;
else if (News1=="Average Earnings Index +Bonus q/y") 
  r1=5;
else if (News1=="Average Hourly Earnings m/m") 
  r1=6;
else if (News1=="Average Weekly Earnings q/q") 
  r1=7;
else if (News1=="BBC Quarterly Economic Survey") 
  r1=8;
/*
"Beige Book"
BIS Meeting
BOC Interest Rate Statement
BOE Interest Rate Decision
BOE Interest Rate Statement
BOE Meeting Minutes
BOE Quarterly Inflation Report
BOJ Interest Rate Decision
BOJ Interest Rate Statement
BOJ Meeting Minutes
BOJ Monetary Policy Report
BRC Retail Sales y/y
Broad Liquidity y/y
Building Approvals m/m
Building Consents
Building Permits
Building Permits m/m
Business Climate Indicator
Business Conditions Index
Business Confidence
Business Inventories
Business Inventories m/m
Business Investment q/q (f)
Capacity Utilization Rate
CBI Distributive Trades Survey
CBI Industrial Trends Indicator
CBI Industrial Trends Orders
CGPI m/m
Chicago PMI
CIPS Construction PMI
Claimant Count Change
Coincident Index
Construction PMI
Construction Spending m/m
Construction Work Done q/q
Consumer Confidence
Consumer Confidence Index
Consumer Credit
Consumer Credit m/m
Consumer Price Index m/m
Consumer Price Index m/m (f)
Consumer Price Index q/q
Consumer Price Index y/y (p)
Consumer Sentiment
Consumer Sentiment (f)
Consumer Sentiment (p)
Consumption Indicator
Core CPI m/m
Core CPI y/y
Core Durable Goods Orders m/m
Core Machinery Orders m/m
Core PCE Price Index m/m
Core PPI m/m
Core Retail Sales m/m
Core Tokyo CPI m/m
Corporate Goods Price Index m/m
Corporate Services Price Index m/m
CPI excl. Food and Energy m/m
CPI excl. Fresh Food m/m
CPI excl. Volatile Items m/m
CPI m/m
CPI m/m (f)
CPI m/m (p)
CPI q/q
CPI y/y
CPI y/y (p)
Crude Oil Inventories
Current Account
Current Account 2Q
Domestic Capital Goods Price Index m/m
Domestic CGPI m/m
Domestic Vehicle Sales
Durable Goods Orders
Durable Goods Orders m/m
Dwelling Commencements q/q
ECB Interest Rate Decision
ECB Interest Rate Statement
Eco Watchers Survey
EIA Crude Oil Inventories
Empire Manufacturing Index
Empire State Business Conditions Index (released early)
Employment
Employment Change
Employment Change q/q
Employment Cost Index q/q
EU Current Account
EU Industrial New Orders m/m
EU Trade Balance
EU ZEW Economic Sentiment
Existing Home Sales
Existing Home Sales m/m
Export Prices m/m
Factory Orders m/m
Factory Orders m/m (p)
FOMC Interest Rate Statement
FOMC Meeting Begins
FOMC Meeting Minutes
Food Price Index m/m
FPI m/m
French Consumer Spending m/m
French Nonfarm Employment Change q/q
French Unemployment Rate
G-20 Meeting Begins
GDP Annualized q/q
GDP Annualized q/q (f)
GDP Deflator Annualized
GDP Deflator Annualized q/q
GDP Deflator Annualized q/q (f)
GDP Deflator q/q
GDP Deflator q/q (f)
GDP Deflator q/q (p)

*/  
  return (r1);
}

